from enum import Enum


class MMLUTask(Enum):
    HIGH_SCHOOL_EUROPEAN_HISTORY = "high_school_european_history"
    BUSINESS_ETHICS = "business_ethics"
    CLINICAL_KNOWLEDGE = "clinical_knowledge"
    MEDICAL_GENETICS = "medical_genetics"
    HIGH_SCHOOL_US_HISTORY = "high_school_us_history"
    HIGH_SCHOOL_PHYSICS = "high_school_physics"
    HIGH_SCHOOL_WORLD_HISTORY = "high_school_world_history"
    VIROLOGY = "virology"
    HIGH_SCHOOL_MICROECONOMICS = "high_school_microeconomics"
    ECONOMETRICS = "econometrics"
    COLLEGE_COMPUTER_SCIENCE = "college_computer_science"
    HIGH_SCHOOL_BIOLOGY = "high_school_biology"
    ABSTRACT_ALGEBRA = "abstract_algebra"
    PROFESSIONAL_ACCOUNTING = "professional_accounting"
    PHILOSOPHY = "philosophy"
    PROFESSIONAL_MEDICINE = "professional_medicine"
    NUTRITION = "nutrition"
    GLOBAL_FACTS = "global_facts"
    MACHINE_LEARNING = "machine_learning"
    SECURITY_STUDIES = "security_studies"
    PUBLIC_RELATIONS = "public_relations"
    PROFESSIONAL_PSYCHOLOGY = "professional_psychology"
    PREHISTORY = "prehistory"
    ANATOMY = "anatomy"
    HUMAN_SEXUALITY = "human_sexuality"
    COLLEGE_MEDICINE = "college_medicine"
    HIGH_SCHOOL_GOVERNMENT_AND_POLITICS = "high_school_government_and_politics"
    COLLEGE_CHEMISTRY = "college_chemistry"
    LOGICAL_FALLACIES = "logical_fallacies"
    HIGH_SCHOOL_GEOGRAPHY = "high_school_geography"
    ELEMENTARY_MATHEMATICS = "elementary_mathematics"
    HUMAN_AGING = "human_aging"
    COLLEGE_MATHEMATICS = "college_mathematics"
    HIGH_SCHOOL_PSYCHOLOGY = "high_school_psychology"
    FORMAL_LOGIC = "formal_logic"
    HIGH_SCHOOL_STATISTICS = "high_school_statistics"
    INTERNATIONAL_LAW = "international_law"
    HIGH_SCHOOL_MATHEMATICS = "high_school_mathematics"
    HIGH_SCHOOL_COMPUTER_SCIENCE = "high_school_computer_science"
    CONCEPTUAL_PHYSICS = "conceptual_physics"
    MISCELLANEOUS = "miscellaneous"
    HIGH_SCHOOL_CHEMISTRY = "high_school_chemistry"
    MARKETING = "marketing"
    PROFESSIONAL_LAW = "professional_law"
    MANAGEMENT = "management"
    COLLEGE_PHYSICS = "college_physics"
    JURISPRUDENCE = "jurisprudence"
    WORLD_RELIGIONS = "world_religions"
    SOCIOLOGY = "sociology"
    US_FOREIGN_POLICY = "us_foreign_policy"
    HIGH_SCHOOL_MACROECONOMICS = "high_school_macroeconomics"
    COMPUTER_SECURITY = "computer_security"
    MORAL_SCENARIOS = "moral_scenarios"
    MORAL_DISPUTES = "moral_disputes"
    ELECTRICAL_ENGINEERING = "electrical_engineering"
    ASTRONOMY = "astronomy"
    COLLEGE_BIOLOGY = "college_biology"
